function [e,pruf]= calculate_eigenvalue(varargin)
%@LPM\CALCULATE_EIGENVALUE
% (1) [e,pruf]= calculate_eigenvalue(e1,e2,phi2,part,vmin)
% or:
% (2) [e,pruf]= calculate_eigenvalue(e,part,vmin)Blatt iteration procedure.
%    (1): first an initial guess is generated by the bisection method.
%    (2): the start point for the iteration is already given in the
%    input (varargin{1})
%
%     e1 / e2     - in input: lower and upper limits for the eigenvalue e_n 
%                 - in output: sharper lower and upper limits
%     part        - data concerning the partition of the equation domain.
%     tol         - the requested tolerance.
%     vmin /vmax  - approximations of the min. and max. of the potential 
%
%      pruf       - the Pruefer theta/pi corresponding to the eigenvalue e.

global schrod_object
x_min = get(schrod_object,'min');
[yi,yf] = get_initConditions;
if nargin == 5
  %bisection is used to obtain good initial estimation
   e = bisection(varargin{1},varargin{2},varargin{3},varargin{4},yi,yf,x_min);
   part = varargin{4};
   vmin = varargin{5};
else
    e = varargin{1};
    part = varargin{2};
    vmin = varargin{3};
end
number_interval=length(part.h);
truncb = number_interval;
trunca = 1;
%Blatt
e0=e;
de0=inf;
itb=0;
im=min(part.imatch,truncb-1);


while true
   de0prev=de0;
   %de0=blatt(e0,yi,yf,part);
   [suml,yl]=propagate_solution_blatt(e0,yi,trunca,im,part,true);
   [sumr,yr]=propagate_solution_blatt(e0,yf,im+1,truncb,part,false);
   cmat=yl(1)/yr(1);
   if ~isinf(cmat)
     yr(2)=yr(2)*cmat;
     sumr=sumr*cmat^2;
   end
   de0=real(yl(1)*(yl(2)-yr(2))/(suml+sumr));  
   if isnan(de0)
        error('OVERFLOW')
    end
   itb=itb+1;
   if abs(de0)>abs(de0prev)
       de0=blatt(e0,yi,yf,part,trunca,truncb);
       if abs(de0)>abs(de0prev)
              if itb<5 && abs(de0prev)>1e-4
                e0=e0-0.9*de0prev;
                de0=blatt(e0,yi,yf,part,trunca,truncb);
              else
                e = ecor;
                break;
              end
       end
   end
    ecor=e0+de0;
    if abs(de0)>max(1e-15*abs(ecor),eps)
             e0 = ecor;
    else
              e = ecor;
              break;
    end
end

pruf = locate_eigenvalue(e,part,vmin);


function [e,e0,e1]=bisection(e0,e1,phi1,bp,yi,yf,x_min)
it=0;
truncb=length(bp.h);
im=min(bp.imatch,truncb-1);
tol0=1e-4;
trunca=1;
while true 
    e=(e0+e1)/2;
    it = it+1;  
    if it >= 70
        break;
    end
	tol = tol0*max(1e-7,abs(e));
    %forward:
    yl = propagate_solution(e,yi,trunca,im,bp,true);
    %backward:
    yr = propagate_solution(e,yf,im+1,truncb,bp,false);
    %the data obtained at each side of xmatch are combined:
    phi = yl(1)*yr(2)-yr(1)*yl(2);
    if phi*phi1>0
        phi1=phi;
        e1=e;
    else
        e0=e;
    end
    if abs(e1-e0)< tol  %min(max((tol*abs(e0)),1e-10),1e-4) 
        break;
    end
 end %while
 
 function d=blatt(e,yi,yf,bp,trunca,truncb)
[suml,yl]=propagate_solution_blatt(e,yi,trunca,bp.imatch,bp,true);
[sumr,yr]=propagate_solution_blatt(e,yf,bp.imatch+1,truncb,bp,false);
while abs(yl(1))<1e-2 && abs(yl(2))>1e-1
    bp.imatch=bp.imatch+2;
    if bp.imatch>truncb
        break;
    end
    [suml,yl]=propagate_solution_blatt(e,yi,trunca,bp.imatch,bp,true);
    [sumr,yr]=propagate_solution_blatt(e,yf,bp.imatch+1,truncb,bp,false);
end
cmat=yl(1)/yr(1);
if ~isinf(cmat)
  yr(2)=yr(2)*cmat;
  sumr=sumr*cmat^2;
end
d=real(yl(1)*(yl(2)-yr(2))/(suml+sumr));



function [sum,y] = propagate_solution_blatt(e,y,imin,imax,bp,forward)
if forward
 t = imin:imax;
else
 t = imax:-1:imin;    
end
ht=bp.h;
sum=0;
if ~isempty(t)
 Tm=get_propagation_matrices(e,bp,t,forward);
 for j = 1:length(t)
    h=ht(t(j));
    sint=y(1)*y(1);
    T=Tm(:,:,j);
    %y = T*y;
    tmp2 = T*y;
    if abs(tmp2(1))>10^50 || abs(tmp2(2))>10^50 || isnan(tmp2(1))
        %avoid overflow
        m = max(abs(y));
        if m == 0
            m = 10^50;
        end
        y = y/m;
        sint=sint/m^2;
        sum=sum/m^2;
        tmp2 = T*y;
    end
    y = tmp2;
    sint=sint+y(1)*y(1);
    sum=sum+sint*h*0.5; %trapezoidal rule
 end
end



